==> libetpan_0.36.orig.tar.gz: libetpan-0.36/src/data-types/mapping.c <==
/*
 * libEtPan! -- a mail stuff library
 *
 * Copyright (C) 2001, 2005 - DINH Viet Hoa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the libEtPan! project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * $Id: mapping.c,v 1.6 2004/11/21 21:53:31 hoa Exp $
 */

#include "mapping.h"

#include <unistd.h>
#include <sys/mman.h>

int get_mapping(size_t length, int prot, int flags,
		int fd, off_t offset,
		void ** presult, void ** pmapping, size_t * pmapping_size)
{
  void * mapping;
  size_t mapping_size;
  void * result;
  size_t page_size;
  off_t delta;

  page_size = getpagesize();
  delta = offset % page_size;

  mapping = mmap(NULL, length + offset, prot, flags, fd, offset - delta);
  if (mapping == MAP_FAILED)
    return -1;

  result = ((char *) mapping) + delta;

  mapping_size = length + offset;

  * pmapping = mapping;
  * pmapping_size = mapping_size;
  * presult = result;

  return 0;
}

==> libetpan_0.48.orig.tar.gz: libetpan-0.48/src/data-types/mapping.c <==
/*
 * libEtPan! -- a mail stuff library
 *
 * Copyright (C) 2001, 2005 - DINH Viet Hoa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the libEtPan! project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * $Id: mapping.c,v 1.7 2006/06/26 11:50:26 hoa Exp $
 */

#include "mapping.h"

#include <unistd.h>
#include <sys/mman.h>

int get_mapping(size_t length, int prot, int flags,
		int fd, off_t offset,
		void ** presult, void ** pmapping, size_t * pmapping_size)
{
  void * mapping;
  size_t mapping_size;
  void * result;
  size_t page_size;
  off_t delta;

  page_size = getpagesize();
  delta = offset % page_size;

  mapping = mmap(NULL, length + offset, prot, flags, fd, offset - delta);
  if (mapping == (void *)MAP_FAILED)
    return -1;

  result = ((char *) mapping) + delta;

  mapping_size = length + offset;

  * pmapping = mapping;
  * pmapping_size = mapping_size;
  * presult = result;

  return 0;
}
